/* ---------------------------------------------------------------------------
Programme : OPERTETE.C
Langage   : C sur UNIX
Date      : Janvier 1996 (Revision)

Auteur    : Francois Meunier

   Ce module comprend les operations relatives a la tete d'une image.
--------------------------------------------------------------------------- */

#include "image.h"
#include <limits.h>

/* ---------------------------------------------------------------------------
Fonction LIRETETE : Cette fonction lit la tete (header) d'un fichier image
                    source (selon le descripteur 'fde').  Le parametre 'info'
permet l'affichage ou non de l'information contenue dans la tete du fichier.
Il y a aussi verification du 'magic number', du type de fichier 'raster' et
du type de la table des couleurs.

Valeurs possible du parametre 'info' :

        0 : n`affiche pas les informations de la tete (header)
        1 : affiche les informations de la tete (header)

appel : lireTete (image, 0 ou 1)
--------------------------------------------------------------------------- */
    void lireTete (Image *image, int info)
   {
   
      if ((image->tete = (TeteRaster *) malloc (sizeof (TeteRaster))) ==
      (TeteRaster *) NULL)
      {
         printf ("\nInfo : Memoire non disponible pour traitement (lireTete)...");
         exit(1);
      
      } /* if then */
   
      if (fread (image->tete, sizeof(TeteRaster), 1, image->fichiers.fde) != 1)
      {
         printf ("\nInfo : Impossible de lire la tete de '%s'.\n",
            *image->fichiers.fichierS);
         exit (-1);
      
      } /* if then */
   
   
   /* Convert from big endian to little endian
   ==================================================================================*/
      image->tete->rasMagic		 = switch_high_low_order(image->tete->rasMagic);
      image->tete->rasLargeur		 = switch_high_low_order(image->tete->rasLargeur);
      image->tete->rasHauteur		 = switch_high_low_order(image->tete->rasHauteur);
      image->tete->rasProfondeur	 = switch_high_low_order(image->tete->rasProfondeur);
      image->tete->rasLongueur	 = switch_high_low_order(image->tete->rasLongueur);
      image->tete->rasType			 = switch_high_low_order(image->tete->rasType);
      image->tete->rasMapType		 = switch_high_low_order(image->tete->rasMapType);
      image->tete->rasMapLongueur = switch_high_low_order(image->tete->rasMapLongueur);
   
   
      if (info)
      {
         printf ("\n");
         printf ("Magic number               = %x \n", image->tete->rasMagic);
         printf ("Nbr d'echelle : X colonnes = %u \n", image->tete->rasLargeur);
         printf ("Nbr de ligne  : Y rangees  = %u \n", image->tete->rasHauteur);
         printf ("Nbr de bits par pixel      = %u \n", image->tete->rasProfondeur);
         printf ("Longueur de l'image : X*Y  = %u \n", image->tete->rasLongueur);
         printf ("Type du fichier raster     = %u \n", image->tete->rasType);
         printf ("Type de la 'Color Map'     = %u \n", image->tete->rasMapType);
         printf ("Longueur de la 'Color map' = %u \n", image->tete->rasMapLongueur);
         printf ("\n");
      
      } /* if then */
   
      if (image->tete->rasMagic != RAS_MAGIC)
      {
         printf ("\nInfo : Le fichier '%s' n'est pas de type 'SunRaster'.\n",
            *image->fichiers.fichierS);
         exit (-2);
      
      } /* if then */
   
      if (image->tete->rasType != RT_STANDARD)
      {
         printf ("\nInfo : Le fichier '%s' n'est pas de type 'SunRaster'",
            " standard.\n", *image->fichiers.fichierS);
         exit (-3);
      
      } /* if then */
   
      if (image->tete->rasMapType != RMT_EQUAL_RGB)
      {
         printf ("\nInfo : Le fichier '%s' est de type 'SunRaster' sans tables",
            " de couleurs.\n", *image->fichiers.fichierS);
         exit (-4);
      
      } /* if then */
   
   } /* lireTete */

/* ---------------------------------------------------------------------------
Fonction ECRIRETETE : Cette fonction ecrit la tete de l'image au fichier
                      destination (selon le descripteur fds).

appel : ecrireTete (image)
--------------------------------------------------------------------------- */
    void ecrireTete (Image *image)
   {
      if (fwrite (image->tete, sizeof(*image->tete), 1, image->fichiers.fds)
      != 1)
      {
         printf("\nInfo : Impossible d'ecrire la tete de '%s'.\n",
            *image->fichiers.fichierD);
         exit(-8);
      
      } /* if then */
   
   } /* ecrireTete */


    int switch_high_low_order(int number)
   {
      unsigned int	N1 = 0;
      unsigned int	N2 = 0;
      unsigned int	N3 = 0;
      unsigned int	N4 = 0;
   
      unsigned int	number1 = number;
      unsigned int	number2 = number;
      unsigned int	number3 = number;
      unsigned int	number4 = number;
   
      int	n_bits = sizeof(int)/4 * CHAR_BIT;
   
      N1 |= ((number1 << 3*n_bits));
      number2 >>= n_bits;
      number2 <<= 3*n_bits;
      N1 |= number2 >> n_bits;
      number3 <<= n_bits;
      number3 >>= 3*n_bits;
      N1 |= number3 << n_bits;
      N1 |= number4 >> 3*n_bits;
   
      return (int)N1;
   }
